/******************************************************************************************
 c_compat.h         verzia 0.1 			 	18.12.09
 Target CPU:       AVR 8-bit
  
 header zabezpecujuci prenositelnost zdrojovych kodov na rozne kompilery

 autor: EdizonTN     edizontn@gmail.com
 web: www.mikrozone.eu
******************************************************************************************/



//  **************************************************************************************
 #ifdef __ICCAVR__                      // ++++++++ IAR EWAVR
  #define                 _mcpu(x)                  __ATmega##x##__ 
  #include <intrinsics.h>
  #ifndef ENABLE_BIT_DEFINITIONS
   #define                 ENABLE_BIT_DEFINITIONS 
  #endif
  #include <inavr.h>
  #define                 _F_CPU                    0                                                       // undefined !!!
  #define                 _sei                      __enable_interrupt()
  #define                 _cli                      __disable_interrupt()
  #define                 _nop                      __no_operation()
  #define                 _wdr                      NOP                                                     // undefined !!!
  #define                 _sleep                    NOP                                                     // undefined !!!
  #define                 _instrdelay(arg)           __delay_cycles(arg)
  #define                 _COMPILER_STRING          "IAR"
  #define                 _get_fuses(addr)          __AddrToZByteToSPMCR_LPM( (void __flash *) addr, 0x09 )                
  #define                 _SYS_HIGH_FUSE            __AddrToZByteToSPMCR_LPM( (void __flash *) 0x0003, 0x09 )
  #define                 _SYS_LOW_FUSE             __AddrToZByteToSPMCR_LPM( (void __flash *) 0x0000, 0x09 )
  #define                 _SYS_EXT_FUSE             __AddrToZByteToSPMCR_LPM( (void __flash *) 0x0002, 0x09 )
  #define                 _SYS_LOCK_FUSE            __AddrToZByteToSPMCR_LPM( (void __flash *) 0x0001, 0x09 )   
  #define                 _Bool                     _Bool
  #define                 _PROGMEM                  __flash 
  #define                 _pgm_read_byte(x)          (*((uint8_t __flash *)(x))) 
  #define                 _pgm_read_word(x)          (*((uint16_t __flash *)(x)))
  #define                 _EEMEM                    0                                                           // undefined !!!
  #define                 _eeprom_read_byte(ads)    0                                                           // undefined !!!
  #define                 _eeprom_write_byte(ads,x) 0                                                           // undefined !!!
  #define                 _eeprom_write_word(ads,x) 0                                                           // undefined !!!
  #define                 _PRAGMA(x)                _Pragma(#x) 

  #define                 _ISR(IRQ_vect)             _PRAGMA(vector = INT0_vect)  \
                                                     __interrupt void isr_##IRQ_vect(void)
  #define                 _main                     void main(void)                                             // undefined !!!
//  ************************************************************************************** 
 #elif __GNUC__ > 0                     // ++++++++ AVR-GCC AVR Studio
  //#define                 _mcpu(x)                  __AVR_ATmega##x##__
  #include      <util/delay_basic.h>
  #include      <stdio.h>
  #include      <avr\io.h>			
  #include      <avr\interrupt.h>	
  #include      <avr/boot.h>
  #define                 _F_CPU                    F_CPU 
  #define                 _sei                      __asm__ __volatile__ ("sei")
  #define                 _cli                      __asm__ __volatile__ ("cli")
  #define                 _nop                      __asm__ __volatile__ ("nop")
  #define                 _wdr                      __asm__ __volatile__ ("wdr")
  #define                 _sleep                    __asm__ __volatile__ ("sleep")
  #define                 _instrdelay(arg)          _delay_loop_2(arg)
  #define                 _COMPILER_STRING          "GCC " __VERSION__
  #define                 _get_fuses(addr)          boot_lock_fuse_bits_get(addr)
  #define                 _SYS_HIGH_FUSE            boot_lock_fuse_bits_get(GET_HIGH_FUSE_BITS)
  #define                 _SYS_LOW_FUSE             boot_lock_fuse_bits_get(GET_LOW_FUSE_BITS)
  #define                 _SYS_EXT_FUSE             boot_lock_fuse_bits_get(GET_EXTENDED_FUSE_BITS)
  #define                 _SYS_LOCK_FUSE            boot_lock_fuse_bits_get(GET_LOCK_BITS)
  #define                 _Bool                     unsigned int
  #define                 _PROGMEM                  PROGMEM
  #define                 _pgm_read_byte(x)         pgm_read_byte(x) 
  #define                 _pgm_read_word(x)         pgm_read_word(x)
  #define                 _EEMEM                    0                                                           // undefined !!!
  #define                 _eeprom_read_byte(ads)    0                                                           // undefined !!!
  #define                 _eeprom_write_byte(ads,x) 0                                                           // undefined !!!
  #define                 _eeprom_write_word(ads,x) 0                                                           // undefined !!!
  #define                 _PRAGMA(x)                PRAGMA(x)
  #define                 _ISR(IRQ_vect)            ISR(IRQ_vect) 
  #define                 _main                     int main(void)
//  **************************************************************************************  
 #elif SDCC > 0                         // ++++++++ Small Device C Compiler
  #define                 _mcpu(x)                  0                                                          // undefined !!!
  #define                 _F_CPU                    SYSCLK 
  #define                 _sei                      __asm__ __volatile__ ("sei")
  #define                 _cli                      __asm__ __volatile__ ("cli")
  #define                 _nop                      __asm__ __volatile__ ("nop")
  #define                 _wdr                      __asm__ __volatile__ ("wdr")
  #define                 _sleep                    __asm__ __volatile__ ("sleep")
  #define                 _instrdelay(arg)          __delay_cycles(arg)
  #define                 _COMPILER_STRING          "SDCC"
  #define                 _get_fuses(addr)          0                                                           // undefined !!!
  #define                 _SYS_HIGH_FUSE            0                                                           // undefined !!!
  #define                 _SYS_LOW_FUSE             0                                                           // undefined !!!
  #define                 _SYS_EXT_FUSE             0                                                           // undefined !!!
  #define                 _SYS_LOCK_FUSE            0                                                           // undefined !!!
  #define                 _Bool                     unsigned int
  #define                 _PROGMEM                  code 
  #define                 _pgm_read_byte(x)         (*((uint8_t code *)(x)))
  #define                 _pgm_read_word(x)         (*((uint16_t code *)(x)))
  #define                 _EEMEM                    0                                                           // undefined !!!
  #define                 _eeprom_read_byte(ads)    0                                                           // undefined !!!
  #define                 _eeprom_write_byte(ads,x) 0                                                           // undefined !!!
  #define                 _eeprom_write_word(ads,x) 0                                                           // undefined !!!
  #define                 _PRAGMA(x)                0                                                           // undefined !!!
  #define                 _ISR(IRQ_vect)            void isr_##IRQ_vect (void) __interrupt (IRQ_vect)
  #define                 _main                     void main(void)                                             // undefined !!!
//  **************************************************************************************
 #elif __IMAGECRAFT__                   // ++++++++ Image Craft CodeVision C ICC
  #define                 _mcpu(x)                  0                                                           // undefined !!! 
  #include      <iccioavr.h> 
  #define                 _F_CPU                    0                                                           // undefined
  #define                 _sei                      asm("sei")
  #define                 _cli                      asm("cli")
  #define                 _nop                      asm("nop")
  #define                 _wdr                      asm("wdr")
  #define                 _sleep                    asm("sleep")
  #define                 _instrdelay(arg)          0                                                           // undefined
  #define                 _COMPILER_STRING          "IMAGECRAFT"
  #define                 _get_fuses(addr)          0                                                           // undefined !!!
  #define                 get_fuses                 asm("ldi r24,0x09")\
                                                    asm("ld r30,y")\
                                                    asm("ldi r31,0x00")\
                                                    asm("sts 0x0068,r24")\
                                                    asm("lpm r30,Z")\
                                                    asm("ret")
  #define                 _SYS_HIGH_FUSE            0                                                           // undefined !!!
  #define                 _SYS_LOW_FUSE             0                                                           // undefined !!!
  #define                 _SYS_EXT_FUSE             0                                                           // undefined !!!
  #define                 _SYS_LOCK_FUSE            0                                                           // undefined !!!
  #define                 _Bool                     unsigned int
  #define                 _PROGMEM                  __flash 
  #define                 _pgm_read_byte(x)         (*((uint8_t __flash *)(x))) 
  #define                 _pgm_read_word(x)         (*((uint16_t __flash *)(x)))
  #define                 _EEMEM                    0                                                           // undefined !!!
  #define                 _eeprom_read_byte(ads)    0                                                           // undefined !!!
  #define                 _eeprom_write_byte(ads,x) 0                                                           // undefined !!!
  #define                 _eeprom_write_word(ads,x) 0                                                           // undefined !!!
  #define                 _PRAGMA(x)                _Pragma( #x ) 
  #define                 _ISR(IRQ_vect)            _PRAGMA(interrupt_handler isr_##IRQ_vect:IRQ_vect) void isr_##IRQ_vect(void) 
  #define                 _main                     void main(void)                                             // undefined !!!
//  **************************************************************************************  
 #elif __CODEVISIONAVR__                // ++++++++ HP InfoTech Code Vision AVR
  #include  "c_codevision.h"
  #define                 _mcpu(x)                  _CHIP_ATXXXXX_ 
  #define                 _F_CPU                    _MCU_CLOCK_FREQUENCY_
  #define                 _sei                      #asm("sei")
  #define                 _cli                      #asm("cli")
  #define                 _nop                      #asm("nop")
  #define                 _wdr                      #asm("wdr")
  #define                 _sleep                    #asm("sleep")
  #define                 _instrdelay(arg)          delay_ms
  #define                 _COMPILER_STRING          "CODEVISION __CODEVISIONAVR__"
  
  //#define                 get_fuses__(addr)         #asm("ldi r24,0x09"); #asm("ld r30,y"); #asm("ldi r31,0x00"); #asm("sts 0x0068,r24"); #asm("lpm r30,Z");#asm("ret");
  #define                 get_fuses__               #asm("ldi r24,0x09"); #asm("ldi r30,0"); #asm("ldi r31,0x00"); #asm("sts 0x0068,r24"); #asm("lpm r30,Z";#asm("ret");  
  #define                 _get_fuses(addr)          get_fuses__(addr) 
  #define                 _SYS_HIGH_FUSE            boot_lock_fuse_bits_get(3)
  #define                 _SYS_LOW_FUSE             boot_lock_fuse_bits_get(0)
  #define                 _SYS_EXT_FUSE             boot_lock_fuse_bits_get(2)
  #define                 _SYS_LOCK_FUSE            boot_lock_fuse_bits_get(1)
  #define                 _Bool                     char
  #define                 _PROGMEM                  flash
  #define                 _pgm_read_byte(x)         (*((uint8_t flash *)(x))) 
  #define                 _pgm_read_word(x)         (*((uint16_t flash *)(x)))
  #define                 _EEMEM                    eeprom 
  #define                 _eeprom_read_byte(ads)    (*((uint8_t eeprom *)(ads))) 
  #define                 _eeprom_write_byte(ads,x) (*((uint8_t eeprom *)(ads))) = x 
  #define                 _eeprom_write_word(ads,x) (*((uint16_t eeprom *)(ads))) = x 
  #define                 _PRAGMA(x)                0                                                            // undefined !!!
  #define                 _ISR(IRQ_vect)           interrupt [IRQ_vect] void isr_##IRQ_vect(void)
  #define                 _main                     void main(void)                                             // undefined !!!
//  **************************************************************************************
 #elif __CROSSWORKS                     // ++++++++ Rowley CrossWorks C
  #include      <inavr.h>
  #include    <cross_studio_io.h>
  #define                 _mcpu(x)                  0                                                           // undefined !!!
  #define                 _F_CPU                    0                                                           // undefined !!!
  #define                 _sei                      __enable_interrupt()
  #define                 _cli                      __disable_interrupt()
  #define                 _nop                      __no_operation()
  #define                 _wdr                      __wdr()
  #define                 _sleep                    __sleep()
  #define                 _instrdelay(arg)          __delay_cycles(arg)
  #define                 _COMPILER_STRING          "CROSSWORKS"
  #define                 _get_fuses(addr)          0                                                           // undefined !!!
  #define                 _SYS_HIGH_FUSE            0                                                           // undefined !!!
  #define                 _SYS_LOW_FUSE             0                                                           // undefined !!!
  #define                 _SYS_EXT_FUSE             0                                                           // undefined !!!
  #define                 _SYS_LOCK_FUSE            0                                                           // undefined !!!
  #define                 _Bool                     unsigned char
  #define                 _PROGMEM                  0                                                           // undefined !!!
  #define                 _pgm_read_byte(x)         0                                                           // undefined !!!
  #define                 _pgm_read_word(x)         0                                                           // undefined !!!
  #define                 _EEMEM                    0                                                           // undefined !!!
  #define                 _eeprom_read_byte(ads)    0                                                           // undefined !!!
  #define                 _eeprom_write_byte(ads,x) 0                                                           // undefined !!!
  #define                 _eeprom_write_word(ads,x) 0                                                           // undefined !!!
  #define                 _PRAGMA(x)                0                                                           // undefined !!!
  #define                 _ISR(IRQ_vect)            void isr_##IRQ_vect(void) __interrupt[IRQ_vect]
  #define                 _main                     void main(void)                                             // undefined !!!
//  **************************************************************************************
 #elif __MIKROC_PRO_FOR_AVR__           // ++++++++ mikroC for AVR
  #define                 _mcpu(x)                  0                                                           // undefined !!!
  #define                 _F_CPU                    0                                                           // undefined !!!
  #define                 _sei                      _asm sei
  #define                 _cli                      _asm cli
  #define                 _nop                      _asm nop
  #define                 _wdr                      _asm wdr
  #define                 _sleep                    _asm sleep
  #define                 _instrdelay(arg)          _asm nop            // undefined
  #define                 _COMPILER_STRING          "MIKRO_C"
  #define                 _get_fuses(addr)          0                                                           // undefined !!!
  #define                 _SYS_HIGH_FUSE            0                                                           // undefined !!!
  #define                 _SYS_LOW_FUSE             0                                                           // undefined !!!
  #define                 _SYS_EXT_FUSE             0                                                           // undefined !!!
  #define                 _SYS_LOCK_FUSE            0                                                           // undefined !!!
  #define                 _Bool                     0                                                           // undefined !!!
  #define                 _PROGMEM                  0                                                           // undefined !!!
  #define                 _pgm_read_byte(x)         0                                                           // undefined !!!
  #define                 _pgm_read_word(x)         0                                                           // undefined !!!
  #define                 _EEMEM                    0                                                           // undefined !!!
  #define                 _eeprom_read_byte(ads)    0                                                           // undefined !!!
  #define                 _eeprom_write_byte(ads,x) 0                                                           // undefined !!!
  #define                 _eeprom_write_word(ads,x) 0                                                           // undefined !!!
  #define                 _PRAGMA(x)                0                                                           // undefined !!!
  #define                 _ISR(IRQ_vect)            void isr_##IRQ_vect() org IRQ_vect
  #define                 _main                     void main(void)                                             // undefined !!!     
 #else
  #error "Neznamy kompiler !"
 #endif













 #define SBIT(port, bit_val)                         ((port) |= (1 << (bit_val)))                               // set bit in port
 #define CBIT(port, bit_val)                         ((port) &= ~(1 << (bit_val)))                              // clear bit in port
 #define BIT(bit_val)                                (0x01 << (bit_val))                                        // definicia,mozes pouzit BIT(3) -> vrati 0x08
 #define TBIT(port, bit_val)                         (port) ^= (1 << (bit_val))                                 // toogle bit in port
 #define WBIT(port, bit_val, newbit)                 (newbit) ? SBIT(port, bit_val) : CBIT(port, bit_val)       // write a bit
 #define GBIT(port, bit_val)                         ((port >> bit_val) & 0x01)                                 // get a bit -> result 0 or 1


// rozne bitove nazvy
 #define _ENABLE        (_Bool)         1       // povol
 #define _DISABLE       (_Bool)         0       // zakaz
 #define _TRUE          (_Bool)         1       // Boolean - true
 #define _FALSE         (_Bool)         0       // Boolean - false
 #define _SET           (_Bool)         1       // nastav
 #define _RESET         (_Bool)         0       // zmaz
 #define _FLAG_CLEAR    (_Bool)         1       // zmaz priznak


 #define _OUTPUT        (_Bool)         1       // DDR ako vystup
 #define _INPUT         (_Bool)         0       // DDR ako vstup
 #define _PULL_ON       (_Bool)         1       // Pull-up zapnuty
 #define _PULL_OFF      (_Bool)         0       // Pull-up vypnuty

